/* Mike and Raj's Code */

/* This program takes accelerometer input and 
   saves it to a file on the SD card. */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>
#include <util/delay.h>


//fatfs libraries
#include "fatfs/ff.h"
#include "fatfs/ff.c"
#include "fatfs/mmc.c"
#include "fatfs/diskio.h"

//#include "lcd_lib.h"
//#include "lcd_lib.c"
#include <string.h>


//const int8_t LCD_initialize[] = "LCD Initialized\0";

//int8_t lcd_buffer[15];	// LCD display buffer

//Serial I/O
#include "uart.h"
#include "uart.c"

//this programs header file
#include "sdacc_test.h"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//debug flag
#define DEBUG

//**********************************************************
//timer 0 overflow ISR
ISR (TIMER0_COMPA_vect) 
begin      
    //Decrement the time if not already zero
    if (SDtime1>0)    --SDtime1;
    if (ADtime2>0)  --ADtime2;
    if (LEDtime3>0) --LEDtime3;
    if (Buttontime4>0) --Buttontime4;
    if (Buzzertime5>0) --Buzzertime5;
	if (Flagstime6>0) --Flagstime6;
	if (LCDtime7>0) --LCDtime7;
	if(READBACK) { if(ReplayTime8>0) --ReplayTime8; }

	if(!READBACK) {
    millis++;
	if(millis == 1000) {
		secs++;
		millis = 0;
		if(secs == 60) {
			mins++;
			secs = 0;
			if(mins == 60) {
				hours++;
				mins = 0;
				if(hours == 24) { hours = 0; }
			}
		}
	}
    
	} //end if(!READBACK)

	if(alarm_armed) {
        //the current time is equal to the alarm time so set buzzer_flag high
        if(hours == wake_hours &&
            mins == wake_mins &&
            secs == wake_secs) { 
                
				buzzer_flag = 1;
                if(READBACK) {
					stopped = 1;
				}
            }
    }
	else if(!alarm_armed && buzzer_flag) {
		buzzer_flag = 0;
	}

    /* check for buzzer flag, if it is high, then set off alarm via PWM */
    if(buzzer_flag) {
        //use PWM here
		PORTD = PORTD | 0x80;
    }
    
    
	/* ------- Disk Timer ---------- */
	if(disk_timer > 0) {
		disk_timer--;
	}
	else if(disk_timer == 0) {
		/*
		Need to call disk_timerproc every 10ms so that the
		SD card knows how long it has been waiting for a
		response from the card 
		*/
		disk_timerproc();
		disk_timer = DISKTIME;
	}
end  

//**********************************************************       
//Entry point
int main(void)
begin  
    initialize();
    init_fs(); //init disk and mount filesystem
	if(!READBACK) { openSleepLog(); } //open the sleep log file
	openMeasurementFile(); //open the measurement file
    // main task scheduler loop 
    while(1)
    begin
		if(!READBACK) {
        	if (SDtime1==0){SDtime1=t1;	writeSDcard();}
        	if (ADtime2==0){ADtime2=t2; takeADmeasurement();}
			if (LEDtime3==0){LEDtime3=t3; updateLEDdisplay();}
			if (Buttontime4==0) {Buttontime4=t4; buttonStateMachine();}
        	if (Buzzertime5==0) {Buzzertime5=t5; setOffBuzzer();}
			if (Flagstime6==0) {Flagstime6=t6; handleButtonFlags();}
			//if (LCDtime7==0) {LCDtime7=t7; LCDtask();}
		}
		else if(!stopped) { //READBACK is 100 times faster
			if (LEDtime3==0){LEDtime3=rt3; updateLEDdisplay();}
			//if (Buttontime4==0) {Buttontime4=rt4; buttonStateMachine();}
        	if (Buzzertime5==0) {Buzzertime5=rt5; setOffBuzzer();}
			//if (Flagstime6==0) {Flagstime6=rt6; handleButtonFlags();}
			//if (LCDtime7==0) {LCDtime7=rt7; LCDtask();}
			if( ReplayTime8==0) { ReplayTime8=rtime1; replayTask(); }
		}
    end
end  
  
//********************************************************** 

/* Reads 10-bit value from the ADC and stores it in global variable reading */
/* Also runs one step of the algorithm */
void takeADmeasurement(void)
{
    //set previous reading before taking new reading
    prev_reading = reading;

	if(!READBACK) {
    //read in low before reading in high
	AinL = ADCL;
	AinH = ADCH;
	reading = 0;
	reading = AinH << 8;
	reading += AinL;

	}
	else {
		readMeasurement();
	}

    /* run algorithm on new reading if we aren't in snooze mode
     * snooze defines the wake-up time if it is in that mode, so we turn off
     * the algorithm in that mode
     */
    if(!snooze_flag) { 
        algorithm_step();
    }

	if(!READBACK) {
 	//start another conversion
 	ADCSRA |= (1<<ADSC) ;
	} 
}

//********************************************************** 
//Set it all up
void initialize(void)
begin
    //DDRD = (1 << PORTD2); //set output for LED on prototype board

    //set up timer 0 for 1 mSec timebase 
    TIMSK0= (1<<OCIE0A);	//turn on timer 0 cmp match ISR 
    OCR0A = 249;  		//set the compare reg. to 250 time ticks
    //set prescalar to divide by 64 
    TCCR0B= 3; //0b00001011;	
    // turn on clear-on-match
    TCCR0A= (1<<WGM01) ;
  
    //initialize ADC
    //init the A to D converter 
    //channel zero/ left adj /EXTERNAL Aref
   	//!!!CONNECT Aref jumper!!!!
   	//ADMUX = (1<<ADLAR) ;  
	ADMUX = 0;     // set up 10-bit conversion

   	//enable ADC and set prescaler to 1/128*16MHz=125,000
   	//and clear interupt enable
   	//and start a conversion
   	ADCSRA = (1<<ADEN) | (1<<ADSC) + 7 ; 

    //init the task timer
    SDtime1=t1;
    ADtime2=t2;
	LEDtime3 = t3;
	Buttontime4=t4;
    Buzzertime5=t5;
	Flagstime6=t6;
	LCDtime7=t7;
	ReplayTime8=rtime1;    

    //init AD reading
    reading = 0;
    prev_reading = 0;
    AinH = 0;
    AinL = 0;
    
    //init time
	millis = 0;
	secs = 0;
	mins = 0;
	hours = 13;
	day = 0; //dummy day
	month = 0; //dummy month
	year = 1; //dummy year
    
    //init flags
    openFlag = 0;
    diskInitFlag = 0;
    diskMountFlag = 0;
    
    counter = 0;
	fileCounter = 0;
    
    //init Algorithm variables
    sampleCounter = 0;
    numMovements = 0;
    movementPeriod = 0;
    numMoveSamples = 0;
    numIntervals = 0;
    lastInterval = 0;
    
    //alarm and wake up flags
    clock = 1;
	alarm_armed = 0;
	buzzer_flag = 0;
    snooze_flag = 0;
    
    //alarm and wake up times
    alarm_secs = 0;
    alarm_mins = 1;
    alarm_hours = 15;
    wake_secs = 0;
    wake_mins = 1;
    wake_hours = 15;
    
	if(READBACK) {
    	//init the task timer
    	SDtime1=t1;
    	ADtime2=t2;
		LEDtime3 = t3;
		Buttontime4=t4;
    	Buzzertime5=t5;
		Flagstime6=t6;
		LCDtime7=t7; 

		secs = 0;
		mins = 0;
		hours = 0;		

		alarm_secs = 0;
		alarm_mins = 0;
		alarm_hours = 7;
		wake_secs = 0;
		wake_mins = 0;
		wake_hours = 7;

		alarm_armed = 1;
	
		stopped = 0;
	}

	//set output for LED display
	PORTLED = 0xff;
	DDRLED = 0xff;

	//set input of Buttons
	DDRD = 0x80;
	PORTD = 0b01111100;

	DDRB = 0x03;
	PORTB = 0x03;

	DDRA = (1<<PINA1);
	PORTA = PORTA | (1<<PINA0);
	

	//init push button states
	minutesButtonPushState = NoPush;
	hoursButtonPushState = NoPush;
	alarmButtonPushState = NoPush;
	snoozeButtonPushState = NoPush;
    
    //init button flags
    minutesButtonFlag = 0;
    hoursButtonFlag = 0;
    alarmButtonFlag = 0;
    snoozeButtonFlag = 0;

    //init the UART -- uart_init() is in uart.c
    uart_init();
    stdout = stdin = stderr = &uart_str;
    #ifdef DEBUG
    fprintf(stdout,"Starting sdacc test...\n\r");
    #endif


	//LCDinit();	//initialize the display
	//LCDcursorOFF();
	//LCDclr();				//clear the display
	//LCDGotoXY(0,0);
	//LCDstring(LCD_initialize, strlen(LCD_initialize));


    //crank up the ISRs
    sei();

end  
//==================================================

/* Inits filesystem on the SD card */
void init_fs() {
	DSTATUS dstatus;
    
    //PORTD &= ~(1 << PORTD2); // LED on
    
	dstatus = disk_initialize(0);

    #ifdef DEBUG
	fprintf(stdout, "Disk Initiliazation: %d\n\r", dstatus);
	if(dstatus == STA_NOINIT) fprintf(stdout, "STA_NOINIT ");
	else if(dstatus == STA_NODISK) fprintf(stdout, "STA_NODISK ");
	else if(dstatus == STA_PROTECT) fprintf(stdout, "STA_PROTECT ");
    #endif
	
    if(dstatus == 0) diskInitFlag = 1; //Disk has been Initialized
	
	FRESULT result;
	//init pointer
	fsPtr = &fs;	
	//set up file system
	result = f_mount(0, fsPtr);
    #ifdef DEBUG
	fprintf(stdout, "Initiating filesystem mount: %d\n\r", result);
    #endif
    
    if(result == 0) diskMountFlag = 1; //Disk has been mounted
    
    //PORTD |= (1 << PORTD2); //LED off
}

/* User Provided Timer Function for FatFs module */
/* DWORD get_fattime() required for writing to SD card */
/*
	From FatFs specification:
	The return value: DWORD
		31:25 - Year from 1980 (0..127)
		24:21 - Month (1..12)
		20:16 - Day in month (1..31)
		15:11 - Hour (0..23)
		10:5 - Minute (0.59)
		4:0 - Second / 2 (0..29)

*/
DWORD get_fattime()
{
	
	DWORD dword;
	//return a made up date, but a sort of right time
	dword = (((DWORD)(year)) << 25)
		 | (((DWORD)(month)) << 21)
		 | (((DWORD)(day)) << 16)
		 | (((DWORD)(hours)) << 11)
		 | (((DWORD)(mins)) << 5)
		 | (((DWORD)(secs)) << 1);

    //fprintf(stdout, "dword %d", dword);
	return dword;
}

/*
Other user provided functions:
(provided in mmc.c)

DSTATUS disk_initialize
DSTATUS disk_status
DRESULT disk_read
DRESULT disk_write
DRESULT disk_ioctl

*/

//Opens the file to write to
void openSleepLog(void) {
	FRESULT fileResult;
	char sleeplogname[26];
    
    //only try to open if the disk has been initialized and mounted
    if(diskInitFlag && diskMountFlag) {
		
		do {
        	snprintf(sleeplogname, 26, "sleep%03d.txt", fileCounter);
            #ifdef DEBUG
        	fprintf(stdout, sleeplogname);
            #endif
        	fileResult = f_open(&logFile, sleeplogname, FA_WRITE | FA_CREATE_NEW);
            #ifdef DEBUG
        	fprintf(stdout, "Open Sleep Log %d\n\r", fileResult);
            #endif
		}
		while(fileResult == FR_EXIST && ++fileCounter < 1000); //deal with existing files on SD card
		
		/*
		snprintf(sleeplogname, 26, "sleeplog.txt");
        fprintf(stdout, sleeplogname);
        fileResult = f_open(&logFile, sleeplogname, FA_WRITE | FA_CREATE_NEW);
        fprintf(stdout, "Open Sleep Log %d\n\r", fileResult);		
		*/
	
        if(fileResult) {
            #ifdef DEBUG
            fprintf(stdout, "File Opening Failed: ");
            printFRcode(fileResult);
            #endif
        }
        else {
            #ifdef DEBUG
            fprintf(stdout, "File Opened.\n\r");
            #endif
            openFlag = 1;
        }
    }
    else {
        #ifdef DEBUG
        fprintf(stdout, "Disk not mounted/initialized can't open file\n\r");
        #endif
    }
}

//Closes the file
void closeSleepLog(void) {
    //only try to close if the file has been opened
    if(openFlag) {
        FRESULT fileResult;
        fileResult = f_close(&logFile);
        if(fileResult) {
            #ifdef DEBUG
            fprintf(stdout, "File Closing Failed: ");
            printFRcode(fileResult);
            #endif
        }
        else {
            #ifdef DEBUG
            fprintf(stdout, "File Close.\n\r");
            #endif
            openFlag = 0;
        }
    }	
}

//Print out file result code
void printFRcode(FRESULT fResult) {
	switch(fResult) {
		case FR_OK:
			fprintf(stdout, "FR OK\n\r");
			break;
		case FR_DISK_ERR:
			fprintf(stdout, "FR Disk Error\n\r");
			break;
		case FR_INT_ERR:
			fprintf(stdout, "FR_INT_ERR\n\r");
			break;
		case FR_NOT_READY:
			fprintf(stdout, "FR_NOT_READY\n\r");
			break;
		case FR_NO_FILE:
			fprintf(stdout, "FR_NO_FILE\n\r");
			break;			
		case FR_NO_PATH:
			fprintf(stdout, "FR_NO_PATH\n\r");
			break;			
		case FR_INVALID_NAME:
			fprintf(stdout, "FR_INVALID_NAME\n\r");
			break;	
		case FR_DENIED:
			fprintf(stdout, "FR_DENIED\n\r");
			break;			
		case FR_EXIST:
			fprintf(stdout, "FR_EXIST\n\r");
			break;			
		case FR_INVALID_OBJECT:
			fprintf(stdout, "FR_INVALID_OBJECT\n\r");
			break;	
		case FR_WRITE_PROTECTED:
			fprintf(stdout, "FR_WRITE_PROTECTED\n\r");
			break;	
		case FR_INVALID_DRIVE:
			fprintf(stdout, "FR_INVALID_DRIVE\n\r");
			break;	
		case FR_NOT_ENABLED:
			fprintf(stdout, "FR_NOT_ENABLED\n\r");
			break;		
		case FR_NO_FILESYSTEM:	
			fprintf(stdout, "FR_NO_FILESYSTEM\n\r");
			break;
		case FR_MKFS_ABORTED:	
			fprintf(stdout, "FR_MKFS_ABORTED\n\r");
			break;
		case FR_TIMEOUT:
			fprintf(stdout, "FR_TIMEOUT\n\r");
			break;			
		default:
			fprintf(stdout, "Unknown FR code\n\r");
			break;
	}
}

//Write ADC reading to SD card
void writeSDcard(void) {
    FRESULT fileResult;
    char str[20];
    unsigned int length;
    unsigned int bytesWrote;
    
    //Write string to file
    snprintf(str, 20, "%02d:%02d:%02d %04d\n\r", hours,
                                                    mins,
                                                    secs,
                                                    reading);
    #ifdef DEBUG
    fprintf(stdout, str); //for debugging
	
	//fprintf(stdout, "%d\n\r", AMPMflag);
    #endif
    
    //PORTD &= ~(1 << PORTD2); // LED on
    
    /* check to see if file is open
    should only write to file if it is open */
    if(openFlag && READBACK) {
		length = strlen(str);
		fileResult = f_write(&logFile, str, length, &bytesWrote);
        //sync filesystem
		f_sync(&logFile);
		
        #ifdef DEBUG
		//fprintf(stdout, "Trying to write to file: %d\n\r", fileResult);
        #endif		
		
		if(!fileResult && (length == bytesWrote)) {
            #ifdef DEBUG
        	//fprintf(stdout, "Successfully written\n\r");
            #endif
			counter++;
		}
    }
    else if(!openFlag && !READBACK) {
        #ifdef DEBUG
        //fprintf(stdout, "Unable to write to file since file is not open\n\r");
        #endif
    }
    
    //PORTD |= (1 << PORTD2); //LED off
}

/* Algorithm **************************************************/
//One step of the algorithm
void algorithm_step(void) {
    signed int dev; //prev_reading - reading
    
    unsigned long mil_time, mil_alarm;
    
    signed long diff; //number of seconds until the alarm time
	signed long diffh, diffm, diffs;
	int period;
    signed long start_window; //diff - ALARM_WINDOW

    signed long iter_diff; //temp variable used for iterating
    unsigned long secsMovePeriod; //movement period in seconds

	
	int hhours, mmins, ssecs; //for sync
	int aalarm_hours, aalarm_mins, aalarm_secs; //for sync

	hhours = hours; mmins = mins; ssecs = secs;
	aalarm_hours = alarm_hours; aalarm_mins = alarm_mins; aalarm_secs = alarm_secs;

    //check to see if we have reached the end of an interval
    if(sampleCounter == (INTERVAL_WINDOW * 4)) {

		#ifdef DEBUG
		//fprintf(stdout, "### end of interval\n\r");
		#endif

        //check to see if threshold for movements in an interval was reached
        if(numMovements >= NUM_MOVEMENTS) {
            //increment the number of intervals that have exceeded NUM_MOVEMENTS threshold
            numMoveSamples++;
            
			period = numIntervals - lastInterval;
           lastInterval = numIntervals;
			
            //calculate the most recent period and average it into the movement period
            //movementPeriod = (int)(((long)((movementPeriod * (numMoveSamples - 1)) 
            //                    + period)) 
            //                               / ((long)numMoveSamples));
			movementPeriod = movementPeriod + (period / numMoveSamples);
            secsMovePeriod = movementPeriod * (long)(INTERVAL_WINDOW * 60 * 4); //4 samples per second
                                                                        
            //if the user wants to be woken up, calculate the wake up time
            if(alarm_armed) {
				fprintf(stdout, "\n\ralgorithm step\n\r");
				fprintf(stdout, "Alarm time: %d:%d:%d\n\r", aalarm_hours, aalarm_mins, aalarm_secs);

                //first calculate how far away the alarm time is from now.
                //in calculating this, we assume the alarm time is always further away
                mil_time = ((long)hhours * 10000L) + ((long)mmins * 100L) + ssecs;
                mil_alarm = ((long)aalarm_hours * 10000L) 
								+ ((long)aalarm_mins * 100L) + aalarm_secs;
                if(mil_alarm < mil_time) {
                    mil_alarm += 240000L;
                }
                
                diffs = mil_alarm % 100 - mil_time % 100;
                diffm = (mil_alarm/100) % 100 - (mil_time/100) % 100;
                diffh = (mil_alarm/10000L) - (mil_time/10000L);
                
                if(diffs < 0) {
                    diffm = diffm - 1;
                    diffs = 60 + diffs;
                }
                if(diffm < 0) {
                    diffh = diffh - 1;
                    diffm = 60 + diffm;
                }
                diff = diffh*3600L + diffm*60 + diffs;
				
                //now calculate ALARM_WINDOW before the alarm time
                start_window = diff - ALARM_WINDOW * 60;
				
				fprintf(stdout, "mil_time: %ld, mil_alarm: %ld\n\r", mil_time, mil_alarm);
				fprintf(stdout, "diff: %ld, start_window: %ld\n\r", diff, start_window);
                
				if(start_window < 0) { //we are inside the alarm time
                    /* and one more addition of the movement period will set off
                       the alarm, then set the alarm time to be now */
                    if( (hhours*3600L + mmins*60 + ssecs) + secsMovePeriod > diff) {
                        wake_hours = hhours;
                        wake_mins = mmins;
                        wake_secs = ssecs;
                    }
                    /* else set the wake up time to be one period later */
                    else {
                        iter_diff = 0;
                        iter_diff = secsMovePeriod;
                        wake_secs = ssecs + (iter_diff % 3600L) % 60L;
                        if(wake_secs >= 60L) {
                                wake_secs = wake_secs % 60L;
                                wake_mins = mmins + 1L;
                        }
                        wake_mins = mmins + (iter_diff % 3600L) / 60L;
                        if(wake_mins >= 60L) {
                                wake_mins = wake_mins % 60L;
                                wake_hours = hhours + 1L;
                        }
                        wake_hours = hhours + (iter_diff / 3600L);
                        if(wake_hours >= 24L) {
                                wake_hours = wake_hours % 24L;
                        }
                    }

                }
                else if(diff > 0 && secsMovePeriod > 0) {
                    /*increment in movementPeriods until you are either
                        past the alarm time or
                        you are inside the alarm time */
                    iter_diff = 0;
                    while(iter_diff < start_window) {
                        iter_diff = iter_diff + secsMovePeriod;
                    }
                    
                    /* if you are past the alarm time,
                        set the wake up time to be the alarm time */
                    if(iter_diff >= diff) { //change to if past the alarm time
                        wake_hours = aalarm_hours;
                        wake_mins = aalarm_mins;
                        wake_secs = aalarm_secs;

						#ifdef DEBUG
						fprintf(stdout, "CCurrent time: %d:%d:%d\n\r", hhours, mmins, ssecs);
						fprintf(stdout, "### movement period %d\n\r", movementPeriod);
						fprintf(stdout, "### secs move period %ld\n\r", secsMovePeriod);
						fprintf(stdout, "### period %d\n\r", period);
						fprintf(stdout, "### iterdiff %ld\n\r", iter_diff);
						fprintf(stdout, "### wake time set to be alarm time: %d:%d:%d\n\r", wake_hours, wake_mins, wake_secs);
						#endif
                    }
                    else {
                        /* if you are inside the alarm time window,
                            loop until you can no longer add another movementPeriod.
                            Set the wake up time to the last time within the alarm window */
                        if(iter_diff < diff && iter_diff >= start_window) {
                            while(iter_diff < diff && iter_diff >= start_window) {
                                iter_diff += secsMovePeriod;
                            }
                            //exits this while loop when iter_diff is past the alarm time
                            iter_diff -= secsMovePeriod; //subtracting one period should bring us back into this window
                            //iter_diff is now the number of seconds until wake up time
                            wake_secs = ssecs + (iter_diff % 3600L) % 60L;
                            if(wake_secs >= 60L) { 
                                wake_secs = wake_secs % 60L;
                                wake_mins = mmins + 1L;
                            }
                            wake_mins = mmins + (iter_diff % 3600L) / 60L;
                            if(wake_mins >= 60L) {
                                wake_mins = wake_mins % 60L;
                                wake_hours = hhours + 1L;
                            }
                            wake_hours = hhours + (iter_diff / 3600L);
                            if(wake_hours >= 24L) {
                                wake_hours = wake_hours % 24L;
                            }
                        }

						#ifdef DEBUG
						fprintf(stdout, "CCurrent time: %d:%d:%d\n\r", hhours, mmins, ssecs);
						fprintf(stdout, "### movement period %d\n\r", movementPeriod);
						fprintf(stdout, "### secsMovePeriod %ld\n\r", secsMovePeriod);
						fprintf(stdout, "### period %d\n\r", period);
						fprintf(stdout, "### iterdiff %ld\n\r", iter_diff);
						fprintf(stdout, "### wake time set: %d:%d:%d\n\r", wake_hours, wake_mins, wake_secs);
						#endif
                    }
                }
            }
        }
        
        //increment interval counter
        numIntervals++;
        
        //reset the interval variables
        sampleCounter = 0;
        numMovements = 0;
    }
    
	sampleCounter++;
    //calculate derivative
    //dev = prev_reading - reading;
    dev = reading-prev_reading;
	
    //check to see if the most recent sample has exceed the threshold
    if(dev < 0 && dev < -MOVE_MAGNITUDE) {
        numMovements++;
    }
    else if(dev >= 0 && dev > MOVE_MAGNITUDE) {
        numMovements++;
    }
}

//Resets the algorithms variables
void algorithm_reset(void) {
    sampleCounter = 0;
    numMovements = 0;
    movementPeriod = 0;
    numMoveSamples = 0;
    numIntervals = 0;
    lastInterval = 0;
}

/* Interfacing LED Display ************************/
void updateLEDdisplay(void) { //NOTE to Raj: task3 from clock.c
    //digits to be displays on LED display
    unsigned char minL, minH, hourL, hourH;
    
    //set digits to be displayed
	if(clock) {
		minL = mins % 10;
		minH  = mins / 10;
		if(hours < 12) {
			if(hours == 0) {
				hourL = 2;
				hourH = 1;
			}
			else {
				hourL = hours % 10;
				hourH  = hours / 10;
			}
		}
		else {
			if( (hours-12) == 0) {
				hourL = 2;
				hourH = 1;
			}
			else {
				hourL = (hours - 12) % 10;
				hourH = (hours - 12) / 10;
			}
		}
	} else {
		minL = alarm_mins % 10;
		minH  = alarm_mins / 10;
		if(alarm_hours < 12) {
			if(alarm_hours == 0) {
				hourL = 2;
				hourH = 1;
			}
			else {
				hourL = alarm_hours % 10;
				hourH  = alarm_hours / 10;
			}
		}
		else {
			if( (alarm_hours-12) == 0) {
				hourL = 2;
				hourH = 1;
			}
			else {
				hourL = (alarm_hours - 12) % 10;
				hourH = (alarm_hours - 12) / 10;
			}
		}
	}

	
	#ifdef DEBUG
	//fprintf(stdout, "%d %d %d %d\n\r", hourH, hourL, minH, minL);
	#endif

	if(!READBACK) {
	PORTLED = 0x70 | minL;
	_delay_us(10);
	PORTLED = 0xb0 | minH;
	_delay_us(10);
	PORTLED = 0xd0 | hourL;
	_delay_us(10);
	PORTLED = (hourH) ? 0xe0 | hourH : 0xef;
	}
	else {
	/*
    if (digit==3) { 
		PORTLED = 0x70 | minL;
		digit = 0;
	} else {
		if (digit==2) {
			//PORTLED = 0xb0 | hourL;
			PORTLED = 0xb0 | minH;
		} else if (digit==1) { 
			//PORTLED = 0xd0 | minH;
			PORTLED = 0xd0 | hourL;
		} else { 
			//PORTLED = 0xe0 | minL; }
			PORTLED = (hourH) ? 0xe0 | hourH : 0xef; }
		digit++;
	}*/

	PORTLED = 0x70 | minL;
	_delay_us(2);
	PORTLED = 0xb0 | minH;
	_delay_us(2);
	PORTLED = 0xd0 | hourL;
	_delay_us(2);
	PORTLED = (hourH) ? 0xe0 | hourH : 0xef;
	}
	
	// minL 0x70
	// minH 0xb0
	// hourL 0xd0
	// hourH 0xe0
	 
	
    //AMPMflag
    if(clock) {
        if(AMPMflag) {
            //set AM/PM LED high
			//PORTB = PORTB & 0xfe;
			PORTA = PORTA & 0xfd;
			//PORTA = PORTA | 0x02;
			
        }
        else {
            //set AM/PM LED low
			//PORTB = PORTB | 0x01;
			PORTA = PORTA | 0x02;
			//PORTA = PORTA & 0xfd;
        }
    }
    else {
        if(AMPMflag) {
            //set AM/PM LED high
			//PORTB = PORTB | 0x01;
			PORTA = PORTA & 0xfd;
			//PORTA = PORTA | 0x02;
        }
        else {
            //set AM/PM LED low
				PORTA = PORTA | 0x02;
			//PORTA = PORTA & 0xfd;
			
        }
    }
	 
}

void buttonStateMachine(void) { //NOTE to Raj: task5 from clock.c
    //minutes button
    switch (minutesButtonPushState)
    {
    case NoPush: 
        if (~PIND & 0x04) minutesButtonPushState=MaybePush;
        else minutesButtonPushState=NoPush;
        break;
    case MaybePush:
        if (~PIND & 0x04) 
        {
           minutesButtonPushState=Pushed;   
           minutesButtonFlag=1;
        }
        else minutesButtonPushState=NoPush;
        break;
     case Pushed:
	 	if (~PIND & 0x04) minutesButtonPushState=Pushed; 
        else minutesButtonPushState=MaybeNoPush;    
        break;
     case MaybeNoPush:
        if (~PIND & 0x04) minutesButtonPushState=Pushed; 
        else 
        {
           minutesButtonPushState=NoPush;
           minutesButtonFlag=0;
        }    
        break;
    }
    
    //hours button
      switch (hoursButtonPushState)
  begin
     case NoPush: 
        if (~PIND & 0x08) hoursButtonPushState=MaybePush;
        else hoursButtonPushState=NoPush;
        break;
     case MaybePush:
        if (~PIND & 0x08) 
        begin
           hoursButtonPushState=Pushed;   
           hoursButtonFlag=1;
        end
        else hoursButtonPushState=NoPush;
        break;
     case Pushed:
	 	if (~PIND & 0x08) hoursButtonPushState=Pushed; 
        else hoursButtonPushState=MaybeNoPush;    
        break;
     case MaybeNoPush:
        if (~PIND & 0x08) hoursButtonPushState=Pushed; 
        else 
        begin
           hoursButtonPushState=NoPush;
           hoursButtonFlag=0;
        end    
        break;
  end
    
	/*
    //alarm button
    switch (alarmButtonPushState)
  begin
     case NoPush: 
        if (~PIND & 0x10) alarmButtonPushState=MaybePush;
        else alarmButtonPushState=NoPush;
        break;
     case MaybePush:
        if (~PIND & 0x10) 
        begin
           alarmButtonPushState=Pushed;   
           alarmButtonFlag=1;
        end
        else alarmButtonPushState=NoPush;
        break;
     case Pushed:
	 	if (~PIND & 0x10) alarmButtonPushState=Pushed; 
        else alarmButtonPushState=MaybeNoPush;    
        break;
     case MaybeNoPush:
        if (~PIND & 0x10) alarmButtonPushState=Pushed; 
        else 
        begin
           alarmButtonPushState=NoPush;
           alarmButtonFlag=0;
        end    
        break;
  end
	*/
    
    //snooze button, check to see if alarm_armed is low, only then should we raise snooze_flag
    switch (snoozeButtonPushState)
  begin
     case NoPush: 
        if (~PIND & 0b00100000) snoozeButtonPushState=MaybePush;
        else snoozeButtonPushState=NoPush;
        break;
     case MaybePush:
        if (~PIND & 0b00100000) 
        begin
           snoozeButtonPushState=Pushed;   
           //if(alarm_armed && buzzer_flag) { snoozeButtonFlag=1; }
        	snoozeButtonFlag = 1;
		end
        else snoozeButtonPushState=NoPush;
        break;
     case Pushed:
	 	if (~PIND & 0b00100000) snoozeButtonPushState=Pushed; 
        else snoozeButtonPushState=MaybeNoPush;    
        break;
     case MaybeNoPush:
        if (~PIND & 0b00100000) snoozeButtonPushState=Pushed; 
        else 
        begin
           snoozeButtonPushState=NoPush;
           snoozeButtonFlag=0;
        end    
        break;
  end


    //check to see if clock is being set
    clock = ~PIND & 0x40;
	alarm_armed = ~PIND & 0b00010000;
}

//Sets buzzerFlag
void setOffBuzzer(void) {
/*
    if(!alarm_armed) {
        //the current time is equal to the alarm time so set buzzer_flag high
        if(hours == wake_hours &&
            mins == wake_mins &&
            secs == wake_secs) { 
                buzzer_flag = 1;
                #ifdef DEBUG
                fprintf(stdout, "BUZZER FLAG SET\n\r");
                #endif 
            }
    }
*/
	


	#ifdef DEBUG
	//fprintf(stdout, "Wakeup time: %d:%d:%d\n\r", wake_hours, wake_mins, wake_secs);
	#endif

	#ifdef DEBUG
	if(buzzer_flag) {
		fprintf(stdout, "BUZZER going off\n\r");
		fprintf(stdout, "Time: %d:%d:%d\n\r", hours, mins, secs);
	}
	#endif
}


void handleSnoozeFlag(void) {
	#ifdef DEBUG
	if(snooze_flag) { fprintf(stdout, "Snooze button\n\r"); }
	#endif

    if(alarm_armed && buzzer_flag && snooze_flag) {
        //set the alarm time to be SNOOZE_TIME mins later
        wake_mins += SNOOZE_TIME;
        if(wake_mins >= 60) {
            wake_mins = wake_mins % 60;
            wake_hours++;
            if(wake_hours >= 24) {
                wake_hours = 0;
            }
        }
        
        snooze_flag = 0;
        buzzer_flag = 0;
		
		#ifdef DEBUG
        fprintf(stdout, "Snoozing for %d mins\n\r", SNOOZE_TIME);
        #endif
    }
}

//handles hours, minutes, and alarm button flag
void handleButtonFlags(void) {
    //increment the minutes button
    if(minutesButtonFlag) {
		#ifdef DEBUG
		fprintf(stdout, "Minutes button pressed\n\r");
        #endif
		if(clock) {
			mins++;
			if(mins == 60) {
				mins = 0;
			}
		}
		else {
			alarm_mins++;
			if (alarm_mins == 60) {
                alarm_mins = 0;
			}
		}
		minutesButtonFlag = 0;
	}
    
    //increment the hours button
    if(hoursButtonFlag) {
        #ifdef DEBUG
		fprintf(stdout, "Hours button pressed\n\r");
        #endif
		if(clock) {
			hours++;
			if(hours == 24) { hours = 0; }
		}
		else {
			alarm_hours++;
			if (alarm_hours == 24) {
				alarm_hours = 0;
			}
		}
		hoursButtonFlag = 0;
	}
    
	
	/*

    //change between clock set and alarm set
    //if(alarmButtonFlag) {
    //    alarm_armed = !alarm_armed;
		if(alarm_armed) {
			//reset algorithm before hand just in case as well
			algorithm_reset();
			#ifdef DEBUG
			fprintf(stdout, "Alarm armed\n\r");
			#endif
		}
		else {
			//reset the algorithm every time we set the alarm
			algorithm_reset();
			#ifdef DEBUG
			fprintf(stdout, "Alarm disarmed\n\r");
			#endif
		}
	//	alarmButtonFlag = 0;
		
    //}
	
	*/	

	if(snoozeButtonFlag) {
		if(alarm_armed && buzzer_flag) { 
			snooze_flag = 1;
			#ifdef DEBUG
			fprintf(stdout, "Snooze flag high\n\r");
			#endif
		}
		else { 
			snooze_flag = 0;
			#ifdef DEBUG
			fprintf(stdout, "Snooze flag low\n\r");
			#endif
		}
		snoozeButtonFlag = 0;

		fprintf(stdout, "Snooze Button pushed\n\r");
	}
	
    if(hours >= 0 && hours < 12) {
					AMPMflag = 0;
				}
				else if(hours >= 12 && hours < 24) {
					AMPMflag = 1;
				}
    
	handleSnoozeFlag();
}

//Opens the measuremtn file to read from
void openMeasurementFile(void) {
    FRESULT fileResult;
    char measurename[26];
   
    //only try to open if the disk has been initialized and mounted
    if(diskInitFlag && diskMountFlag) {
   
        snprintf(measurename, 26, "measure.txt");
        #ifdef DEBUG
        fprintf(stdout, measurename);
        #endif
        fileResult = f_open(&measureFile, measurename, FA_READ);
        #ifdef DEBUG
        fprintf(stdout, "Open Sleep Log %d\n\r", fileResult);
        #endif
   
        if(fileResult) {
            #ifdef DEBUG
            fprintf(stdout, "Measurement File Opening Failed: ");
            printFRcode(fileResult);
            #endif
            measureFlag = 0;
        }
        else {
            #ifdef DEBUG
            fprintf(stdout, "Measurement File Opened.\n\r");
            #endif
            measureFlag = 1;
        }
    }
    else {
        #ifdef DEBUG
        fprintf(stdout, "Disk not mounted/initialized can't open file\n\r");
        #endif
    }
}

void readMeasurement(void) {
    int readIn;
	char buffer[8];

	f_gets(buffer, 8, &measureFile);
	
	#ifdef DEBUG
	//fprintf(stdout, "Reading Measurement\n\r");
	//fprintf(stdout, buffer);
	#endif

	readIn = atoi(buffer);

    if(measureFlag) {
        //read line from &measureFile
        //some sprintf string to int parsing...
        //need to mark which line
        reading = readIn;
    }
}


void replayTask(void) {
	prev_reading = reading;
	readMeasurement();
	algorithm_step();

	#ifdef DEBUG
	//fprintf(stdout, "%d:%d:%d %d\n\r", hours, mins, secs, reading);
	#endif

	millis += 250;
	if(millis == 1000) {
		secs++;
		millis = 0;
		if(secs == 60) {
			mins++;
			secs = 0;
			if(mins == 60) {
				hours++;
				mins = 0;
				if(hours == 24) {
					hours = 0;
				}
			}
		}
	}
}
/*
void LCDtask(void) {
	char d3, d2, d1, d0;
	LCDclr();
	if(clock) {
		d0 = mins % 10;
		d1  = mins / 10;
		if(hours < 12) {
			if(hours == 0) {
				d2 = 2;
				d3 = 1;
			}
			else {
				d2 = hours % 10;
				d3  = hours / 10;
			}
		}
		else {
			if( (hours-12) == 0) {
				d2 = 2;
				d3 = 1;
			}
			else {
				d2 = (hours - 12) % 10;
				d3 = (hours - 12) / 10;
			}
		}
	} else {
		d0 = alarm_mins % 10;
		d1  = alarm_mins / 10;
		if(alarm_hours < 12) {
			if(alarm_hours == 0) {
				d2 = 2;
				d3 = 1;
			}
			else {
				d2 = alarm_hours % 10;
				d3  = alarm_hours / 10;
			}
		}
		else {
			if( (alarm_hours-12) == 0) {
				d2 = 2;
				d3 = 1;
			}
			else {
				d2 = (alarm_hours - 12) % 10;
				d3 = (alarm_hours - 12) / 10;
			}
		}
	}
	if(AMPMflag) {
		snprintf(lcd_buffer, 15, "%1d%1d:%1d%1d:%d (PM)\n\r", d3,d2,d1,d0,secs);
	}
	else {
		snprintf(lcd_buffer, 15, "%1d%1d:%1d%1d:%d (AM)\n\r", d3,d2,d1,d0,secs);
	}
	LCDclr();
	LCDstring(lcd_buffer, strlen(lcd_buffer));
}
*/
