/* Mike and Raj's Code */
//header file for sdacc_test.c


//time for tasks
#define t1 250
#define t2 250
#define t3 40 //how often we update LED display
#define t4 50 //Button state machine  
#define t5 250
#define t6 200
#define t7 200



#define begin {
#define end }

/* Functions *******************************************/
/* Function that writes the A/D reading to the SD card */
void writeSDcard(void);

/* Function that takes A/D measurement and stores it in global variable reading */
void takeADmeasurement(void);

//Algorithm signatures
void algorithm_step(void); //one step of the algorithm
void algorithm_reset(void); //resets algorithms variables

/* Function that updates all 4 LED Displays and the AM/PM LED */
void updateLEDdisplay(void);

/* State machine determines button states for all of the buttons */
void buttonStateMachine(void);

/* Function that compares the current time to the alarm time.
    If the current time is equal to the alarm time
    and the alarm is set, it should set the buzzer flag high.
    The buzzer is set off in the ISR while checking the buzzer flag. 
    This should be run faster than 1 ms (faster than the ISR). */
void setOffBuzzer(void);

/* Deals with the flags that were set during buttonStateMachine
    Handles the hours, minutes, and alarm button flags */
void handleButtonFlags(void);

/* Handles the snooze flag. The snooze flag is only set high if the alarm
    is set. We do an additional check on the alarm set flag here. Buzzer flag
    also needs to be high since we only want to snooze if the buzzer is going off.
    If the snooze flag is high, then add the snooze amount to the
    alarm time. Then set the snooze flag low. */
void handleSnoozeFlag(void);

/* Function that initiates the file system */
void init_fs(void);

/* Opens/creates the sleep log for writing values to.
   Sets the file handle */
void openSleepLog(void);

/* Function that maps file result codes to
   output strings */
void printFRcode(FRESULT fResult);

/* Function that initializes all microcontroller stuff */
void initialize(void);



//timeout counters        
volatile unsigned char SDtime1; 
volatile unsigned char ADtime2;
volatile unsigned char LEDtime3;
volatile unsigned char Buttontime4;
volatile unsigned char Buzzertime5;
volatile unsigned char Flagstime6;
volatile unsigned char LCDtime7;

//ADC
unsigned int reading; //accelerometer reading
unsigned int prev_reading; //previous accelerometer reading
char AinH, AinL; //high and low registers for ADC

//time
int millis;
unsigned char secs;
unsigned char mins;
unsigned char hours;
unsigned char day;
unsigned char month;
unsigned char year; 

//disk timer
#define DISKTIME 10 //10ms timer
unsigned char disk_timer;

//file system pointers
FATFS fs;
FATFS* fsPtr;
FIL logFile;
FIL measureFile;

unsigned char diskInitFlag;
unsigned char diskMountFlag;

//Number of times the file has been written to
unsigned int counter;

//Number of files on the SD card
unsigned int fileCounter;

unsigned char openFlag; //is the file open?


/* Sleep Data Processing ******************************/
//Algorithm Parameters
#define NUM_MOVEMENTS 3 //threshold for movements in an interval
#define INTERVAL_WINDOW 5 //length of interval in minutes
#define MOVE_MAGNITUDE 7 //threshold for change in accelerometer magnitude

//Algorithm State Variables
unsigned int sampleCounter; //counter of number of samples in the interval
unsigned int numMovements; //Number of Movements in this interval
unsigned int movementPeriod; //Average Movement Period in intervals
unsigned int numMoveSamples; //Number of intervals that have exceeded 
                             // the NUM_MOVEMENTS threshold
unsigned int numIntervals; //Number of Intervals that have elasped
unsigned int lastInterval; //the last interval that had NUM_MOVEMENTS threshold exceeded


/* Alarm and Wake up  ******************************/
//Flags
/* Is the clock time or the alarm time being set?
 * Clock being set: 1
 * Alarm being set: 0
 */
unsigned char clock;

/* Is the alarm armed?
 * No: 0
 * Yes: 1
 */
unsigned char alarm_armed;

/* Is the buzzer going off right now?
 * No: 0
 * Yes: 1
 */
unsigned char buzzer_flag;

/* Has the snooze button been pressed? (Are we in snooze mode?)
 * No: 0
 * Yes: 1
 */
unsigned char snooze_flag;

//User set alarm time
unsigned char alarm_secs;
unsigned char alarm_mins;
unsigned char alarm_hours;

//Wake up time specified by the algorithm / snooze
unsigned char wake_secs;
unsigned char wake_mins;
unsigned char wake_hours;

//How many minutes before the alarm time can the user be woken up?
#define ALARM_WINDOW 30 //in minutes


/* Interfacing Buttons *********************/
/* Flag to indicate whether or not it's AM or PM
 * AM: 0
 * PM: 1
 */
volatile unsigned char AMPMflag;

//State machine state names
#define NoPush 1 
#define MaybePush 2
#define Pushed 3
#define MaybeNoPush 4

//Button states
unsigned char minutesButtonPushState;
unsigned char hoursButtonPushState;
unsigned char digitButtonPushState;
unsigned char alarmButtonPushState;
unsigned char snoozeButtonPushState;

//button flags
unsigned char minutesButtonFlag, hoursButtonFlag, digitButtonFlag,
                alarmButtonFlag, snoozeButtonFlag;

//How much time do we want to snooze?
#define SNOOZE_TIME 5 //in mins

/* LED Display variables ******************/
//7-segment display mappings
#define sev0 0b01010000	//0x7e
#define sev1 0b01010001	//0x30
#define sev2 0b01010010	//0x6d
#define sev3 0b01010011	//0x79
#define sev4 0b01010100	//0x33
#define sev5 0b01010101	//0x5b
#define sev6 0b01010110	//0x5f
#define sev7 0b01010111	//0x70
#define sev8 0b01011000	//0x7f
#define sev9 0b01011001	//0x7b

//output port for 7-segment display
#define PORTLED PORTC 
#define DDRLED DDRC

unsigned char digit; //to Raj: what is this for?

/* Read back mode ************************/
#define READBACK 1
#define READBACK_RATE 10 //how many milliseconds in a second?

//time for tasks
#define rt1 2
#define rt2 2
#define rt3 8 //how often we update LED display
#define rt4 10 //Button state machine  
#define rt5 2 //check buzzer
#define rt6 40
#define rt7 200 //LCD time

/* The measurement file */
FIL measureFile;

/* measure file opened flag */
unsigned char measureFlag;

/* Sets the measurement File */
void openMeasurementFile(void);

/* Reads a measurement from the SD card **/
void readMeasurement(void);

/* Is readback stopped? */
unsigned char stopped;

void replayTask(void);
#define rtime1 2
unsigned char ReplayTime8;
