%% plot the data and noise margin
M = csvread('data');
M = M(1:100800);
figure;
% scale to hours
% 1 hour = 14400 samples (4 samples/sec)
hold on
plot(1/14400:1/14400:7, M(1:100800), 'o');
title('Raw Accelerometer Output', 'FontSize', 16);
xlabel('Time (hours)', 'FontSize', 16);
ylabel('ADC value', 'FontSize', 16);


%% the actual algorithm

% plot the derivative
dev = ones(1, length(M) - 1);
for i = 2:length(M)
    dev(i) = M(i) - M(i-1);
end

figure;
hold on
plot(1/14400:1/14400:length(dev)/14400, dev,'o');
title('Normalized Accelerometer Output', 'FontSize', 16);
xlabel('Time (hours)', 'FontSize', 16);
ylabel('ADC value', 'FontSize', 16);
hold off

%%

% parameters of algorithm
numberMovements = 3;
intervalWindow = 5;
movementMagnitude = 7; % how big the derivative is

% number of movements of |derivative| > 10 in 5 min window
% 5 mins in samples = 20 samples

sampleCounter = 0; 
intervalCounter = 0;
movementPeriod = 0; % average period (in intervals)
numMoveSamples = 0; % number of movement samples
numMovements = 0; % number of movements in this interval
firstInterval = 1; % first interval flag

lastInterval = 0;

intervalVector = zeros(1, ceil(length(dev)/(intervalWindow * 60 * 4)));
numIntervals = 0;

% want the period (in 5 min intervals) of the number of movements
% in those intervals > 3

a_time = 7;
alarm_time = (a_time) * 14400;
w_time = 30;
window = w_time * 60 * 4;
start_window = alarm_time - window;

wakeuptime = ones(1,length(dev));

period_vector = ones(1,30);

for i = 1:length(dev)
        
    
    % stuff to do at the end of each interval
    if(sampleCounter == intervalWindow * 60 * 4) % intervalWindow * 4 samples/sec
        if(numMovements >= numberMovements)
            % increment number of movement samples
            numMoveSamples = numMoveSamples + 1;
            
            % calculate the most recent period
            if(firstInterval == 1)
                period = intervalCounter;
                firstInterval = 0;
            else
                period = intervalCounter - lastInterval;
                lastInterval = intervalCounter;
            end
            
            periodVector(numMoveSamples) = period;
            % now average the most recent period
            movementPeriod = ((movementPeriod * (numMoveSamples - 1)) + period) / numMoveSamples;
            
            % convert movement period to samples (from intervals)
            secsMovePeriod = round(movementPeriod) * intervalWindow * 60 * 4;
            
            % calculate how far we are from the alarm time
            diff = alarm_time - i;
            
            % if inside time window and threshold has been reached
            if(i >= start_window && i < alarm_time)
                % and one more addition of the movement period will set off
                % the alarm, then set the alarm time to be now
                if(i + secsMovePeriod > alarm_time)
                    wakeuptime(i) = i;
                else
                    wakeuptime(i) = i + secsMovePeriod;
                end
            elseif(diff > 0 && secsMovePeriod > 0)
            
                % we are inside the alarm time, set off alarm now
                if(start_window < 0)
                    wakeuptime = alarm_time;
                    sprintf('WAKE UP NOW!')
                end
            
                % increment in movement periods until either past the 
                % alarm time or within the alarm window
                iter_diff = i;
                while(iter_diff < start_window)
                    iter_diff = iter_diff +  secsMovePeriod;
                end
                
                % if the iterated time is past the alarm time, set the wake
                % up time to be the alarm time
                if(iter_diff >= alarm_time)
                    wakeuptime(i) = alarm_time;
                else
                    % if you are inside the alarm time,
                    %    loop until you can no longer add another movementPeriod.
                    %   Set the wake up time to the last time within the
                    %   alarm window
                    if(iter_diff < alarm_time && iter_diff >= start_window)
                        while(iter_diff < alarm_time && iter_diff >= start_window)
                            iter_diff = iter_diff + secsMovePeriod;
                        end
                        %% exits this while loop when iter_diff is past the alarm time
                        % subtracting one period should bring us back into
                        % this window
                        iter_diff = iter_diff - secsMovePeriod; 
                        
                        %iter_diff is now wake up time
                        wakeuptime(i) = iter_diff;
                        iter_diff
                    else
                        wakeuptime(i) = alarm_time;
                    end
                end
           
            else
                wakeuptime(i) = alarm_time;
            end
            
        end
        
        %increment intervalCounter
        intervalCounter = intervalCounter + 1;
        
        intervalVector(intervalCounter) = numMovements;
        
        % reset sample Counter
        sampleCounter = 0;
        numMovements = 0;
        % numMoveSamples = 0;
        
        numIntervals = numIntervals + 1;
    end
    
    % count the number of triggers in this interval
    %if(abs(dev(i)) > movementMagnitude)
    %    numMovements = numMovements + 1;
    %end
    if(dev(i) < 0 && dev(i) < -movementMagnitude)
        numMovements = numMovements + 1;
    elseif(dev(i) >= 0 && dev(i) > movementMagnitude)
        numMovements = numMovements + 1;
    end
    
    sampleCounter = sampleCounter + 1;
end

movementPeriod

%% Plot Interval Vector 
figure;
hold on
plot(((1:length(intervalVector)) * (intervalWindow * 60 *4))/14400, intervalVector, 'o')
title('Number of Movements that exceeded MAX MOVEMENT in each interval');
xlabel('Time (hours)');
ylabel('Count');

%% Overlay Accelerometer plot, derivative plot, and interval plot

figure;
% accelerometer subplot
subplot(311)
hold on
plot(1/14400:1/14400:length(M)/14400, M, 'o')
title('Raw Accelerometer Output')
xlabel('Hour')
hold off

% derivative subplot
subplot(312)
hold on
plot(1/14400:1/14400:length(dev)/14400, dev);
title('Derivative of raw Accelerometer Output');
xlabel('Hour')
hold off

% interval plot
subplot(313)
hold on
% plot each interval value in the middle of the real-time
plot(((1:length(intervalVector)) * (intervalWindow*60*4))/14400, intervalVector, 'o')
plot([0 length(intervalVector)*(intervalWindow*60*4)/14400], [numberMovements numberMovements], 'r');
title(strcat('Number of movements in ', num2str(intervalWindow), ' minute intervals'));
hold off



% NOTE: things to consider:
% 2 options of picking wake up time:
%   1) checking for predicted wake up time based on movementPeriod in
%   wake-up time period
%   2) detecting moving based on number of movements in an interval and
%   wake up when this is detected

% Also, should NOT include initial movement from sleeping

%% plot wake up times
figure;
hold on
plot(1/14400:1/14400:length(wakeuptime)/14400, wakeuptime/14400, 'o')
plot([a_time a_time], [a_time 0], '-r');
plot([a_time-w_time/60 a_time-w_time/60], [a_time 0], '-g');

for i = 1:length(wakeuptime)
    if(i == wakeuptime(i))
        plot(i/14400, wakeuptime(i)/14400, 'or');
    end
end

title('Predicted Wake-up Times', 'FontSize', 16);
ylabel('Wake-up Time', 'FontSize', 16);
xlabel('Time', 'FontSize', 16);

hold off

