//Mike and Raj's code
//Code for testing writing to SD card

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include "sd_test.h"
//#include "fatfs/ff.h"
#include "ff.h"
#include "ff.c"
#include "mmc.c"

#include "uart.h"
#include "uart.c"

#include "sd_test.h"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//interrupt every 1 ms
ISR(TIMER0_COMPA_vect)
{
	millis++;
	if(millis == 1000) {
		secs++;
		millis = 0;
		if(secs == 60) {
			mins++;
			secs = 0;
			if(mins == 60) {
				hours++;
				mins = 0;
				if(hours == 24) {
					hours = 0;
				}
			}
		}
	}

	/* ------- Disk Timer ---------- */
	if(disk_timer > 0) {
		disk_timer--;
	}
	else if(disk_timer == 0) {
		/*
		Need to call disk_timerproc every 10ms so that the
		SD card knows how long it has been waiting for a
		response from the card 
		*/
		disk_timerproc();
		disk_timer = DISKTIME;
	}
}

/* User Provided Timer Function for FatFs module */
/* DWORD get_fattime() required for writing to SD card */
/*
	From FatFs specification:
	The return value: DWORD
		31:25 - Year from 1980 (0..127)
		24:21 - Month (1..12)
		20:16 - Day in month (1..31)
		15:11 - Hour (0..23)
		10:5 - Minute (0.59)
		4:0 - Second / 2 (0..29)

*/
DWORD get_fattime()
{
	//return a made up date, but a sort of right time
	return (((DWORD)(year)) << 25)
		 | (((DWORD)(month)) << 21)
		 | (((DWORD)(day)) << 16)
		 | (((DWORD)(hours)) << 11)
		 | (((DWORD)(mins)) << 5)
		 | (((DWORD)(secs)) << 1);
}

/*
Other user provided functions:
(provided in mmc.c)

DSTATUS disk_initialize
DSTATUS disk_status
DRESULT disk_read
DRESULT disk_write
DRESULT disk_ioctl

*/

int main(void) {

	//init
	//set timer 0 for 1 mSec timebase
	TIMSK0 = (1<<OCIE0A);
	OCR0A = 249;
	TCCR0B = 3;
	TCCR0A = (1<<WGM01);

	//init time
	millis = 0;
	secs = 0;
	mins = 0;
	hours = 0;
	day = 0; //dummy day
	month = 0; //dummy month
	year = 0; //dummy year

    ButtonPushState = NoPush;
    ButtonFlag = 0;
    WritingFlag = 0;

	//init the UART -- uart_init() is in uart.c
	uart_init();
	stdout = stdin = stderr = &uart_str;
	fprintf(stdout,"Starting timers...\n\r");

	sei(); //turn on ISRs

	init_fs(); //init disk and mount filesystem
	while(1) {
		
        if(ButtonFlag) { fprintf(stdout, "Butotn Pressed\n\r"); }
        
        if(ButtonFlag && !WritingFlag) {
             //write a file
            FIL file_obj; //file handle
            // open file with write permission
            f_open(&file_obj, "raj.txt", FA_WRITE);
            //write text to file
            f_printf(&file_obj, "raaaaaaaaaaaaaaaj");
            //close file, write cached content
            f_close(&file_obj);
            
            WritingFlag = 1;
            ButtonFlag = 0;
            fprintf(stdout, "Written\n\r");
        }
    }
}

/* Inits the SD card */
void init_fs() {
	disk_initialize(0); 

	FRESULT result;
	//init pointer
	fsPtr = &fs;	
	//set up file system
	result = f_mount(0, fsPtr);
	fprintf(stdout, "Filesystem result: %d\n\r", result);
}

/* Reads for button press */
void task1(void) {
    switch (ButtonPushState) {
     case NoPush: 
        if (~PIND & 0x04) ButtonPushState=MaybePush;
        else ButtonPushState=NoPush;
        break;
     case MaybePush:
        if (~PIND & 0x04) 
        begin
           ButtonPushState=Pushed;   
           ButtonFlag=1;
        end
        else ButtonPushState=NoPush;
        break;
     case Pushed:
	 	if (~PIND & 0x04) ButtonPushState=Pushed; 
        else minutesButtonPushState=MaybeNoPush;    
        break;
     case MaybeNoPush:
        if (~PIND & 0x04) ButtonPushState=Pushed; 
        else 
        {
           ButtonPushState=NoPush;
           ButtonFlag=0;
        }    
        break;
    }
}
